import { JsonRpcProvider, RawSigner, SuiObjectResponse, SuiTransactionBlockResponse, TransactionBlock } from "@mysten/sui.js";
import { SuiError } from "./types";
export declare const executeTransactionBlock: (signer: RawSigner, transactionBlock: TransactionBlock) => Promise<SuiTransactionBlockResponse>;
export declare const getFieldsFromObjectResponse: (object: SuiObjectResponse) => Record<string, any> | null;
export declare const getInnerType: (type: string) => string | null;
export declare const getPackageIdFromType: (type: string) => string | null;
export declare const getTableKeyType: (tableType: string) => string | null;
export declare const getObjectFields: (provider: JsonRpcProvider, objectId: string) => Promise<Record<string, any> | null>;
export declare const getTokenFromTokenRegistry: (provider: JsonRpcProvider, tokenBridgeStateObjectId: string, tokenType: string) => Promise<SuiObjectResponse>;
export declare const getTokenCoinType: (provider: JsonRpcProvider, tokenBridgeStateObjectId: string, tokenAddress: Uint8Array, tokenChain: number) => Promise<string | null>;
/**
 * Get the fully qualified type of a wrapped asset published to the given
 * package ID.
 *
 * All wrapped assets that are registered with the token bridge must satisfy
 * the requirement that module name is `coin` (source: https://github.com/wormhole-foundation/wormhole/blob/a1b3773ee42507122c3c4c3494898fbf515d0712/sui/token_bridge/sources/create_wrapped.move#L88).
 * As a result, all wrapped assets share the same module name and struct name,
 * since the struct name is necessarily `COIN` since it is a OTW.
 * @param coinPackageId packageId of the wrapped asset
 * @returns Fully qualified type of the wrapped asset
 */
export declare const getWrappedCoinType: (coinPackageId: string) => string;
export declare const isSuiError: (error: any) => error is SuiError;
/**
 * This method validates any Sui address, even if it's not 32 bytes long, i.e.
 * "0x2". This differs from Mysten's implementation, which requires that the
 * given address is 32 bytes long.
 * @param address Address to check
 * @returns If given address is a valid Sui address or not
 */
export declare const isValidSuiAddress: (address: string) => boolean;
export declare const isValidSuiType: (type: string) => boolean;
export declare const getPackageId: (provider: JsonRpcProvider, objectId: string) => Promise<string>;
